@@ -1,6 +1,6 @@
 #include <pthread.h>
 
-#include <darwintest.h>
+#include "darwintest_defaults.h"
 
 #define MAX_THREADS 512
 #define THREAD_DEPTH 32
